#include "GLWindow.h"
#include <iostream>
#include <ngl/Camera.h>
#include <ngl/Colour.h>
#include <ngl/Light.h>
#include <ngl/Matrix.h>
#include <ngl/Transformation.h>
#include <ngl/TransformStack.h>
#include <ngl/Material.h>
#include <ngl/NGLInit.h>
#include <ngl/Obj.h>
#include <ngl/Random.h>
#include <ngl/VBOPrimitives.h>
#include <ngl/ShaderManager.h>
#include <boost/foreach.hpp>



//----------------------------------------------------------------------------------------------------------------------
GLWindow::GLWindow(
									int _timer,
									QWidget *_parent

									) :
                    QGLWidget(_parent)
{

  // set this widget to have the initial keyboard focus
  setFocus();
  // re-size the widget to that of the parent (in this case the GLFrame passed in on construction)
  this->resize(_parent->size());
  // Now set the initial GLWindow attributes to default values
  // Roate is false
  m_rotate=false;
  // mouse rotation values set to 0
  m_spinXFace=0;
  m_spinYFace=0;
	m_timerValue=_timer;
	startSimTimer();
}

// This virtual function is called once before the first call to paintGL() or resizeGL(),
//and then once whenever the widget has been assigned a new QGLContext.
// This function should set up any required OpenGL context rendering flags, defining display lists, etc.

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::initializeGL()
{

glClearColor(0.4f, 0.4f, 0.4f, 1.0f);			   // Grey Background
// enable depth testing for drawing
 glEnable(GL_DEPTH_TEST);
 // Now we will create a basic Camera from the graphics library
 // This is a static camera so it only needs to be set once
 // First create Values for the camera position
 ngl::Vector From(0,0,5);
 ngl::Vector To(0,0,0);
 ngl::Vector Up(0,1,0);
 ngl::NGLInit *Init = ngl::NGLInit::instance();
 #ifdef WIN32
	glewInit();
 #endif
 Init->initGlew();
 m_cam= new ngl::Camera(From,To,Up,ngl::PERSPECTIVE);
 // set the shape using FOV 45 Aspect Ratio based on Width and Height
 // The final two are near and far clipping planes of 0.5 and 10
 m_cam->setShape(45,(float)720.0/576.0,0.5,150,ngl::PERSPECTIVE);
 // now to load the shader and set the values
 // grab an instance of shader manager
 ngl::ShaderManager *shader=ngl::ShaderManager::instance();
 // load a frag and vert shaders
 shader->loadShader("Blinn","shaders/Vertex.vs","shaders/Fragment.fs");
 // set this as the active shader
 shader->useShader("Blinn");
 // now pass the modelView and projection values to the shader
 shader->setShaderParamFromMatrix("Blinn","ViewMatrix",m_cam->getModelView());
 shader->setShaderParamFromMatrix("Blinn","projectionMatrix",m_cam->getProjection());

 shader->loadShader("Colour","shaders/Colour.vs","shaders/Colour.fs");

// set this as the active shader
shader->useShader("Colour");
shader->setShaderParamFromMatrix("Colour","ViewMatrix",m_cam->getModelView());
shader->setShaderParamFromMatrix("Colour","projectionMatrix",m_cam->getProjection());

shader->setShaderParam4f("Colour","Colour",1,1,1,1);


 // the shader will use the currently active material and light0 so set them
 ngl::Material m(ngl::SILVER);
 m.use();
 ngl::Light *L1 = new ngl::Light(ngl::Vector(0,0,1,1),ngl::Colour(1,1,1,1),ngl::LIGHTLOCAL);
 L1->enable();

 ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();
 prim->createVBOSphere("sphere",1.0,20);
}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget has been resized.
// The new size is passed in width and height.
void GLWindow::resizeGL(
                        int _w,
                        int _h
                       )
{
  glViewport(0,0,_w,_h);
  m_cam->setShape(45,(float)_w/_h,0.5,150,ngl::PERSPECTIVE);
  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  (*shader)["Blinn"]->use();
	shader->setShaderParamFromMatrix("Blinn","projectionMatrix",m_cam->getProjection());
  (*shader)["Colour"]->use();
	shader->setShaderParamFromMatrix("Colour","projectionMatrix",m_cam->getProjection());

}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget needs to be painted.
// this is our main drawing routine
void GLWindow::paintGL()
{
  // grab an instance of the shader manager
  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  // clear the screen and depth buffer

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  // Rotation based on the mouse position for our global
  // transform
  ngl::Transformation trans;
  trans.setRotation(m_spinXFace,m_spinYFace,0);
  // set this in the TX stack
  m_transformStack.setGlobal(trans);
	// draw spring lines
  shader->useShader("Colour");
	m_transformStack.pushTransform();
	{
		// get our position values and put in a vector
		std::vector<ngl::Vector> points;
		points.push_back(m_spring->getAPosition());
		points.push_back(m_spring->getBPosition());
		// convert to point
		points[0].m_w=1.0;
		points[1].m_w=1.0;
		// load transform stack
		m_transformStack.loadGlobalAndCurrentMatrixToShader("Colour","ModelMatrix");
		glEnableClientState(GL_VERTEX_ARRAY);
		// draw
		glVertexPointer(4, GL_FLOAT, 0, &points[0]);
		glDrawArraysInstancedARB(GL_LINES, 0, points.size(),1);
		glDisableClientState(GL_VERTEX_ARRAY);
	}
	m_transformStack.popTransform();

	// get an instance of the VBO primitives for drawing
  ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();

	shader->useShader("Blinn");



	m_transformStack.pushTransform();
	{
    shader->setShaderParam4f("Blinn","Colour",1.0,0.0,0.0,1.0);
    m_transformStack.setScale(0.1,0.1,0.1);
    m_transformStack.setPosition(m_spring->getAPosition());
    m_transformStack.loadGlobalAndCurrentMatrixToShader("Blinn","ModelMatrix");

    // draw the cube
    prim->draw("cube");
	}
	m_transformStack.popTransform();


	m_transformStack.pushTransform();
	{
    shader->setShaderParam4f("Blinn","Colour",0.0,1.0,0.0,1.0);
    m_transformStack.setScale(0.1,0.1,0.1);
    m_transformStack.setPosition(m_spring->getBPosition());
    m_transformStack.loadGlobalAndCurrentMatrixToShader("Blinn","ModelMatrix");

    // draw the sphere
    prim->draw("cube");
	}
	m_transformStack.popTransform();
	// draw the target points
	shader->setShaderParam4f("Blinn","Colour",1.0,0.0,0.0,1.0);

	m_transformStack.pushTransform();
	{
      shader->setShaderParam4f("Blinn","Colour",1.0,0.0,0.0,1.0);
      m_transformStack.setScale(0.1,0.1,0.1);
			m_transformStack.setPosition(m_spring->getInitialAPosition());
			m_transformStack.loadGlobalAndCurrentMatrixToShader("Blinn","ModelMatrix");

	    // draw the cube
      prim->draw("sphere");
	}
	m_transformStack.popTransform();


	m_transformStack.pushTransform();
	{
      shader->setShaderParam4f("Blinn","Colour",0.0,1.0,0.0,1.0);

			m_transformStack.setScale(0.1,0.1,0.1);
			m_transformStack.setPosition(m_spring->getInitialBPosition());
			m_transformStack.loadGlobalAndCurrentMatrixToShader("Blinn","ModelMatrix");

	    // draw the sphere
      prim->draw("sphere");
	}
	m_transformStack.popTransform();
}





//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mouseMoveEvent (
                               QMouseEvent * _event
                              )
{
  // note the method buttons() is the button state when event was called
  // this is different from button() which is used to check which button was
  // pressed when the mousePress/Release event is generated
  if(m_rotate && _event->buttons() == Qt::LeftButton)
  {
    m_spinYFace = ( m_spinYFace + (_event->x() - m_origX) ) % 360 ;
    m_spinXFace = ( m_spinXFace + (_event->y() - m_origY) ) % 360 ;
    m_origX = _event->x();
    m_origY = _event->y();
  }
  // re-draw GL
  updateGL();
}


//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mousePressEvent (
                                QMouseEvent * _event
                               )
{
  // this method is called when the mouse button is pressed in this case we
  // store the value where the maouse was clicked (x,y) and set the Rotate flag to true
  if(_event->button() == Qt::LeftButton)
  {
    m_origX = _event->x();
    m_origY = _event->y();
    m_rotate =true;
  }
}

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mouseReleaseEvent (
                                  QMouseEvent * _event
                                 )
{
  // this event is called when the mouse button is released
  // we then set Rotate to false
  if (_event->button() == Qt::LeftButton)
  {
    m_rotate=false;
  }
}

void GLWindow::timerEvent(
													 QTimerEvent *_event
												  )
{
	m_spring->update();
	updateGL();
}

GLWindow::~GLWindow()
{
	ngl::NGLInit *init = ngl::NGLInit::instance();
	init->NGLQuit();
}

void GLWindow::startSimTimer()
{

m_timer=startTimer(m_timerValue);
}

void GLWindow::stopSimTimer()
{
 killTimer(m_timer);
}


